@@ -1,7 +1,6 @@
 module Transformer
     module SyskitPlugin
-        def self.compute_required_transformations(manager, task, options = Hash.new)
-            options = Kernel.validate_options options, :validate_network => true
+        def self.compute_required_transformations(manager, task)
             static_transforms  = Hash.new
             dynamic_transforms = Hash.new { |h, k| h[k] = Array.new }
 
@@ -16,17 +15,14 @@ module Transformer
                 if !from || !to
                     # This is validated in #validate_generated_network. Just
                     # ignore here.
-                    #
-                    # We do that so that the :validate_network option to
-                    # Engine#instanciate applies
                     next
                 end
-                
+
                 # Register transformation producers that are connected to
                 # some of our transformation input ports
                 self_producers = Hash.new
-                tr.each_transform_port do |port, transform|
-                    if port.kind_of?(Orocos::Spec::InputPort) && task.connected?(port.name)
+                tr.each_transform_input do |port, transform|
+                    if task.connected?(port.name)
                         port_from = task.selected_frames[transform.from]
                         port_to   = task.selected_frames[transform.to]
                         if port_from && port_to
@@ -53,12 +49,8 @@ module Transformer
                     begin
                         manager.transformation_chain(from, to, self_producers)
                     rescue Exception => e
-                        if options[:validate_network]
-                            raise InvalidChain.new(task, trsf.from, from, trsf.to, to, e),
-                                "cannot find a transformation chain to produce #{from} => #{to} for #{task} (task-local frames: #{trsf.from} => #{trsf.to}): #{e.message}", e.backtrace
-                        else
-                            next
-                        end
+                        raise InvalidChain.new(manager, task, trsf.from, from, trsf.to, to, e),
+                            "cannot find a transformation chain to produce #{from} => #{to} for #{task} (task-local frames: #{trsf.from} => #{trsf.to}): #{e.message}", e.backtrace
                     end
                 Transformer.log_pp(:debug, chain)
 
@@ -146,8 +138,7 @@ module Transformer
         #
         # @return [Boolean] true if producers have been added to the plan and
         #   false otherwise
-        def self.add_needed_producers(tasks, instanciated_producers, options = Hash.new)
-            options = Kernel.validate_options options, :validate_network => true
+        def self.add_needed_producers(tasks, instanciated_producers)
             has_new_producers = false
             tasks.each do |task|
                 dependency_graph = task.relation_graph_for(Roby::TaskStructure::Dependency)
@@ -156,7 +147,7 @@ module Transformer
 
                 Transformer.debug { "computing needed static and dynamic transformations for #{task}" }
 
-                static_transforms, dynamic_transforms = compute_required_transformations(tr_manager, task, :validate_network => options[:validate_network])
+                static_transforms, dynamic_transforms = compute_required_transformations(tr_manager, task)
                 task.static_transforms = static_transforms.values
                 dynamic_transforms.each do |producer_model, transformations|
                     producer_tasks = instanciated_producers[producer_model]
@@ -188,28 +179,26 @@ module Transformer
                 task.requirements.transformer.each_static_transform do |static|
                     static_transforms[[static.from, static.to]] = static
                 end
-                
+
                 tr = task.model.transformer
                 task_name = task.orocos_name
                 tr.each_annotated_port do |port, frame_name|
                     selected_frame = task.selected_frames[frame_name]
                     if selected_frame
-                        info = Types::Transformer::PortFrameAssociation.new(
-                            :task => task_name, :port => port.name, :frame => selected_frame)
+                        info = Types.transformer.PortFrameAssociation.new(
+                            task: task_name, port: port.name, frame: selected_frame)
                         state.port_frame_associations << info
                     elsif Syskit.conf.transformer_warn_about_unset_frames?
                         Transformer.warn "no frame selected for #{frame_name} on #{task}. This is harmless for the network to run, but will make the display of #{port.name} \"in the right frame\" impossible"
                     end
                 end
-                tr.each_transform_port do |port, transform|
-                    next if port.kind_of?(Orocos::Spec::InputPort)
-
+                tr.each_transform_output do |port, transform|
                     from = task.selected_frames[transform.from]
                     to   = task.selected_frames[transform.to]
                     if from && to
-                        info = Types::Transformer::PortTransformationAssociation.new(
-                            :task => task_name, :port => port.name,
-                            :from_frame => from, :to_frame => to)
+                        info = Types.transformer.PortTransformationAssociation.new(
+                            task: task_name, port: port.name,
+                            from_frame: from, to_frame: to)
                         state.port_transformation_associations << info
                     elsif Syskit.conf.transformer_warn_about_unset_frames?
                         Transformer.warn "no frame selected for #{transform.to} on #{task}. This is harmless for the network to run, but might remove some options during display"
@@ -218,7 +207,7 @@ module Transformer
             end
 
             state.static_transformations = static_transforms.values.map do |static|
-                rbs = Types::Base::Samples::RigidBodyState.invalid
+                rbs = Types.base.samples.RigidBodyState.invalid
                 rbs.sourceFrame = static.from
                 rbs.targetFrame = static.to
                 rbs.position = static.translation
@@ -241,14 +230,14 @@ module Transformer
 
             def handle_start_vertex(root_task)
                 @selected_frames = Hash.new
-                selected_frames[root_task] = FramePropagation.initialize_selected_frames(root_task, Hash.new)
                 FramePropagation.initialize_transform_producers(root_task, Transformer::Configuration.new)
+                selected_frames[root_task] = FramePropagation.initialize_selected_frames(root_task, Hash.new)
             end
 
             def handle_examine_edge(from, to)
+                FramePropagation.initialize_transform_producers(to, from.transformer)
                 selected_frames[to] = FramePropagation.initialize_selected_frames(
                     to, selected_frames[from])
-                FramePropagation.initialize_transform_producers(to, from.transformer)
             end
         end
 
@@ -263,36 +252,7 @@ module Transformer
         # with its connections
         #
         # It only checks its inputs, as it is meant to iterate over all tasks
-        def self.validate_frame_selection_consistency_through_inputs(task)
-            task.each_annotated_port do |task_port, task_frame|
-                next if !task_port.input? || !task_frame
-                task_port.each_frame_of_connected_ports do |other_port, other_frame|
-                    if other_frame != task_frame
-                        raise FrameSelectionConflict.new(
-                            task,
-                            task.model.find_frame_of_port(task_port),
-                            task_frame,
-                            other_frame)
-                    end
-                end
-            end
-            task.each_transform_port do |task_port, task_transform|
-                next if !task_port.input?
-                task_port.each_transform_of_connected_ports do |other_port, other_transform|
-                    if other_transform.from && task_transform.from && other_transform.from != task_transform.from
-                        task_local_name = task.model.find_transform_of_port(task_port).from
-                        raise FrameSelectionConflict.new(task, task_local_name,
-                                                         task_transform.from, other_transform.from)
-                    elsif other_transform.to && task_transform.to && other_transform.to != task_transform.to
-                        task_local_name = task.model.find_transform_of_port(task_port).to
-                        raise FrameSelectionConflict.new(task, task_local_name,
-                                                         task_transform.to, other_transform.to)
-                    end
-                end
-            end
-        end
-
-        def self.instanciated_network_postprocessing_hook(engine, plan, validate)
+        def self.instanciated_network_postprocessing_hook(engine, plan)
             needed = true
             all_producers = Hash.new { |h, k| h[k] = Array.new }
             while needed
@@ -302,18 +262,7 @@ module Transformer
 
                 # Now find out the frame producers that each task needs, and add them to
                 # the graph
-                needed = add_needed_producers(transformer_tasks, all_producers, validate_network: engine.options[:validate_abstract_network])
-            end
-
-            # We must now validate. The frame propagation algorithm does
-            # some validation, but also tries to do as little work as
-            # possible and therefore will miss some errors
-            if engine.options[:validate_abstract_network]
-                transformer_tasks = plan.find_local_tasks(Syskit::TaskContext).
-                    find_all { |task| task.model.transformer }
-                transformer_tasks.each do |task|
-                    validate_frame_selection_consistency_through_inputs(task)
-                end
+                needed = add_needed_producers(transformer_tasks, all_producers)
             end
         end
 
@@ -322,7 +271,9 @@ module Transformer
                 find_all { |task| task.model.transformer }
 
             # And update the configuration state
-            update_configuration_state(plan.transformer_configuration_state[1], transformer_tasks)
+            if !Roby.app.testing?
+                update_configuration_state(plan.transformer_configuration_state[1], transformer_tasks)
+            end
             plan.transformer_configuration_state[0] = Time.now
         end
 
@@ -331,11 +282,10 @@ module Transformer
                 if app.testing?
                     Syskit.conf.transformer_warn_about_unset_frames = false
                 end
-
-                Roby.app.using_task_library('transformer')
             end
 
-            def self.require_config(app)
+            def self.require_models(app)
+                app.using_task_library('transformer')
                 Syskit.conf.use_deployment('transformer_broadcaster')
             end
         end
@@ -345,11 +295,13 @@ module Transformer
 
             # Maintain a transformer broadcaster on the main engine
             Roby::ExecutionEngine.add_propagation_handler(description: 'syskit-transformer transformer broadcaster start') do |plan|
-		if Syskit.conf.transformer_broadcaster_enabled?
-		    if !plan.execution_engine.quitting? && plan.find_tasks(OroGen::Transformer::Task).not_finished.empty?
-			plan.add_mission_task(OroGen::Transformer::Task)
-		    end
-		end
+            if Syskit.conf.transformer_broadcaster_enabled?
+                if !plan.execution_engine.quitting? && plan.find_tasks(OroGen.transformer.Task).not_finished.empty?
+                    # The broadcaster will be updated at most once per
+                    # execution cycle
+                    plan.add_mission_task(OroGen.transformer.Task.to_instance_requirements.period(0.05))
+                end
+            end
             end
 
             Syskit::NetworkGeneration::Engine.register_instanciation_postprocessing do |engine, plan|
@@ -358,9 +310,9 @@ module Transformer
                 end
             end
 
-            Syskit::NetworkGeneration::Engine.register_instanciated_network_postprocessing do |engine, plan, validate|
+            Syskit::NetworkGeneration::Engine.register_instanciated_network_postprocessing do |engine, plan|
                 if Syskit.conf.transformer_enabled?
-                    instanciated_network_postprocessing_hook(engine, plan, validate)
+                    instanciated_network_postprocessing_hook(engine, plan)
                 end
             end
 
@@ -394,6 +346,9 @@ module Transformer
             Syskit::Robot::DeviceInstance.class_eval do
                 prepend Transformer::DeviceExtension
             end
+            Syskit::Robot::MasterDeviceInstance.class_eval do
+                prepend Transformer::MasterDeviceExtension
+            end
             Syskit::Graphviz.class_eval do
                 prepend Transformer::GraphvizExtension
             end
@@ -401,12 +356,15 @@ module Transformer
             Syskit::InstanceRequirements.class_eval do
                 prepend Transformer::InstanceRequirementsExtension
             end
-            Syskit::NetworkGeneration::Engine.class_eval do
-                prepend Transformer::EngineExtension
+            Syskit::NetworkGeneration::SystemNetworkGenerator.class_eval do
+                prepend Transformer::SystemNetworkGeneratorExtension
             end
             Syskit::Actions::Profile.class_eval do
                 prepend Transformer::ProfileExtension
             end
+            Syskit::Actions::Profile::RobotDefinition.class_eval do
+                prepend Transformer::ProfileRobotDefinitionExtension
+            end
         end
 
         def self.register
